package galleryimages

import (
	"context"
	"fmt"
	"net/http"
	"net/url"

	"github.com/Azure/go-autorest/autorest"
	"github.com/Azure/go-autorest/autorest/azure"
)

// Copyright (c) Microsoft Corporation. All rights reserved.
// Licensed under the MIT License. See NOTICE.txt in the project root for license information.

type ListByGalleryOperationResponse struct {
	HttpResponse *http.Response
	Model        *[]GalleryImage

	nextLink     *string
	nextPageFunc func(ctx context.Context, nextLink string) (ListByGalleryOperationResponse, error)
}

type ListByGalleryCompleteResult struct {
	Items []GalleryImage
}

func (r ListByGalleryOperationResponse) HasMore() bool {
	return r.nextLink != nil
}

func (r ListByGalleryOperationResponse) LoadMore(ctx context.Context) (resp ListByGalleryOperationResponse, err error) {
	if !r.HasMore() {
		err = fmt.Errorf("no more pages returned")
		return
	}
	return r.nextPageFunc(ctx, *r.nextLink)
}

// ListByGallery ...
func (c GalleryImagesClient) ListByGallery(ctx context.Context, id GalleryId) (resp ListByGalleryOperationResponse, err error) {
	req, err := c.preparerForListByGallery(ctx, id)
	if err != nil {
		err = autorest.NewErrorWithError(err, "galleryimages.GalleryImagesClient", "ListByGallery", nil, "Failure preparing request")
		return
	}

	resp.HttpResponse, err = c.Client.Send(req, azure.DoRetryWithRegistration(c.Client))
	if err != nil {
		err = autorest.NewErrorWithError(err, "galleryimages.GalleryImagesClient", "ListByGallery", resp.HttpResponse, "Failure sending request")
		return
	}

	resp, err = c.responderForListByGallery(resp.HttpResponse)
	if err != nil {
		err = autorest.NewErrorWithError(err, "galleryimages.GalleryImagesClient", "ListByGallery", resp.HttpResponse, "Failure responding to request")
		return
	}
	return
}

// preparerForListByGallery prepares the ListByGallery request.
func (c GalleryImagesClient) preparerForListByGallery(ctx context.Context, id GalleryId) (*http.Request, error) {
	queryParameters := map[string]interface{}{
		"api-version": defaultApiVersion,
	}

	preparer := autorest.CreatePreparer(
		autorest.AsContentType("application/json; charset=utf-8"),
		autorest.AsGet(),
		autorest.WithBaseURL(c.baseUri),
		autorest.WithPath(fmt.Sprintf("%s/images", id.ID())),
		autorest.WithQueryParameters(queryParameters))
	return preparer.Prepare((&http.Request{}).WithContext(ctx))
}

// preparerForListByGalleryWithNextLink prepares the ListByGallery request with the given nextLink token.
func (c GalleryImagesClient) preparerForListByGalleryWithNextLink(ctx context.Context, nextLink string) (*http.Request, error) {
	uri, err := url.Parse(nextLink)
	if err != nil {
		return nil, fmt.Errorf("parsing nextLink %q: %+v", nextLink, err)
	}
	queryParameters := map[string]interface{}{}
	for k, v := range uri.Query() {
		if len(v) == 0 {
			continue
		}
		val := v[0]
		val = autorest.Encode("query", val)
		queryParameters[k] = val
	}

	preparer := autorest.CreatePreparer(
		autorest.AsContentType("application/json; charset=utf-8"),
		autorest.AsGet(),
		autorest.WithBaseURL(c.baseUri),
		autorest.WithPath(uri.Path),
		autorest.WithQueryParameters(queryParameters))
	return preparer.Prepare((&http.Request{}).WithContext(ctx))
}

// responderForListByGallery handles the response to the ListByGallery request. The method always
// closes the http.Response Body.
func (c GalleryImagesClient) responderForListByGallery(resp *http.Response) (result ListByGalleryOperationResponse, err error) {
	type page struct {
		Values   []GalleryImage `json:"value"`
		NextLink *string        `json:"nextLink"`
	}
	var respObj page
	err = autorest.Respond(
		resp,
		azure.WithErrorUnlessStatusCode(http.StatusOK),
		autorest.ByUnmarshallingJSON(&respObj),
		autorest.ByClosing())
	result.HttpResponse = resp
	result.Model = &respObj.Values
	result.nextLink = respObj.NextLink
	if respObj.NextLink != nil {
		result.nextPageFunc = func(ctx context.Context, nextLink string) (result ListByGalleryOperationResponse, err error) {
			req, err := c.preparerForListByGalleryWithNextLink(ctx, nextLink)
			if err != nil {
				err = autorest.NewErrorWithError(err, "galleryimages.GalleryImagesClient", "ListByGallery", nil, "Failure preparing request")
				return
			}

			result.HttpResponse, err = c.Client.Send(req, azure.DoRetryWithRegistration(c.Client))
			if err != nil {
				err = autorest.NewErrorWithError(err, "galleryimages.GalleryImagesClient", "ListByGallery", result.HttpResponse, "Failure sending request")
				return
			}

			result, err = c.responderForListByGallery(result.HttpResponse)
			if err != nil {
				err = autorest.NewErrorWithError(err, "galleryimages.GalleryImagesClient", "ListByGallery", result.HttpResponse, "Failure responding to request")
				return
			}

			return
		}
	}
	return
}

// ListByGalleryComplete retrieves all of the results into a single object
func (c GalleryImagesClient) ListByGalleryComplete(ctx context.Context, id GalleryId) (ListByGalleryCompleteResult, error) {
	return c.ListByGalleryCompleteMatchingPredicate(ctx, id, GalleryImageOperationPredicate{})
}

// ListByGalleryCompleteMatchingPredicate retrieves all of the results and then applied the predicate
func (c GalleryImagesClient) ListByGalleryCompleteMatchingPredicate(ctx context.Context, id GalleryId, predicate GalleryImageOperationPredicate) (resp ListByGalleryCompleteResult, err error) {
	items := make([]GalleryImage, 0)

	page, err := c.ListByGallery(ctx, id)
	if err != nil {
		err = fmt.Errorf("loading the initial page: %+v", err)
		return
	}
	if page.Model != nil {
		for _, v := range *page.Model {
			if predicate.Matches(v) {
				items = append(items, v)
			}
		}
	}

	for page.HasMore() {
		page, err = page.LoadMore(ctx)
		if err != nil {
			err = fmt.Errorf("loading the next page: %+v", err)
			return
		}

		if page.Model != nil {
			for _, v := range *page.Model {
				if predicate.Matches(v) {
					items = append(items, v)
				}
			}
		}
	}

	out := ListByGalleryCompleteResult{
		Items: items,
	}
	return out, nil
}
